﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ServoAdj
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();

            // COMポートの設定
            comPort.setBaudRate(38400);
            comPort.setStxEtx('#', '$');
            comPort.onReceived += onReceived;
            comPort.onOpened += onOpened;
            comPort.onClosed += onClosed;

            textOfs0.Text = trackOfs0.Value.ToString();
            textOfs1.Text = trackOfs1.Value.ToString();
            textOfs2.Text = trackOfs2.Value.ToString();
            textAmp0.Text = trackAmp0.Value.ToString();
            textAmp1.Text = trackAmp1.Value.ToString();
            textAmp2.Text = trackAmp2.Value.ToString();
            textTest0.Text = trackTest0.Value.ToString();
            textTest1.Text = trackTest1.Value.ToString();
            textTest2.Text = trackTest2.Value.ToString();

            textBattery.Text = "";
        }
        
        // リロード
        private void buttonReload_Click(object sender, EventArgs e)
        {
            comPort.sendMessage("AL");
        }

        // セーブ
        private void buttonSave_Click(object sender, EventArgs e)
        {
            comPort.sendMessage("AS");
        }

        // 極性
        private void checkPol0_CheckedChanged(object sender, EventArgs e)
        {
            string val = checkPol0.Checked ? "-" : "+";
            comPort.sendMessage("AP" + val + "0");

            trackTest0.Value = 0;
            trackTest0_Scroll(null, null);
        }
        private void checkPol1_CheckedChanged(object sender, EventArgs e)
        {
            string val = checkPol1.Checked ? "-" : "+";
            comPort.sendMessage("AP" + val + "1");

            trackTest1.Value = 0;
            trackTest1_Scroll(null, null);
        }
        private void checkPol2_CheckedChanged(object sender, EventArgs e)
        {
            string val = checkPol2.Checked ? "-" : "+";
            comPort.sendMessage("AP" + val + "2");

            trackTest2.Value = 0;
            trackTest2_Scroll(null, null);
        }

        // オフセット調整
        private void trackOfs0_Scroll(object sender, EventArgs e)
        {
            int ofs = trackOfs0.Value;
            textOfs0.Text = ofs.ToString();

            if (ofs < 0) ofs += 256;
            string val = ofs.ToString("X2");
            comPort.sendMessage("AO" + val + "0");

            trackTest0.Value = 0;
            trackTest0_Scroll(null, null);
        }
        private void trackOfs1_Scroll(object sender, EventArgs e)
        {
            int ofs = trackOfs1.Value;
            textOfs1.Text = ofs.ToString();

            if (ofs < 0) ofs += 256;
            string val = ofs.ToString("X2");
            comPort.sendMessage("AO" + val + "1");

            trackTest1.Value = 0;
            trackTest1_Scroll(null, null);
        }
        private void trackOfs2_Scroll(object sender, EventArgs e)
        {
            int ofs = trackOfs2.Value;
            textOfs2.Text = ofs.ToString();

            if (ofs < 0) ofs += 256;
            string val = ofs.ToString("X2");
            comPort.sendMessage("AO" + val + "2");

            trackTest2.Value = 0;
            trackTest2_Scroll(null, null);
        }
        
        // 振幅調整
        private void trackAmp0_Scroll(object sender, EventArgs e)
        {
            int amp = trackAmp0.Value;
            textAmp0.Text = amp.ToString();

            string val = amp.ToString("X2");
            comPort.sendMessage("AA" + val + "0");

            trackTest0.Value = 0;
            trackTest0_Scroll(null, null);
        }
        private void trackAmp1_Scroll(object sender, EventArgs e)
        {
            int amp = trackAmp1.Value;
            textAmp1.Text = amp.ToString();

            string val = amp.ToString("X2");
            comPort.sendMessage("AA" + val + "1");

            trackTest1.Value = 0;
            trackTest1_Scroll(null, null);
        }
        private void trackAmp2_Scroll(object sender, EventArgs e)
        {
            int amp = trackAmp2.Value;
            textAmp2.Text = amp.ToString();

            string val = amp.ToString("X2");
            comPort.sendMessage("AA" + val + "2");

            trackTest2.Value = 0;
            trackTest2_Scroll(null, null);
        }

        // テスト
        private void trackTest0_Scroll(object sender, EventArgs e)
        {
            int steer = trackTest0.Value;
            textTest0.Text = steer.ToString();

            if (steer < 0) steer += 256;
            string val = steer.ToString("X2");
            comPort.sendMessage("S" + val + "0");
        }
        private void trackTest1_Scroll(object sender, EventArgs e)
        {
            int steer = trackTest1.Value;
            textTest1.Text = steer.ToString();

            if (steer < 0) steer += 256;
            string val = steer.ToString("X2");
            comPort.sendMessage("S" + val + "1");
        }
        private void trackTest2_Scroll(object sender, EventArgs e)
        {
            int steer = trackTest2.Value;
            textTest2.Text = steer.ToString();
            
            if (steer < 0) steer += 256;
            string val = steer.ToString("X2");
            comPort.sendMessage("S" + val + "2");
        }

        // COMポート受信ハンドラ
        private void onReceived(object sender, ComPortEventArgs e)
        {
            char[] data = e.data;
            string strData = new string(data);

            switch(data[0])
            {
                // サーボ調整
                case 'A':
                    if (data[1] == 'L')
                    {
                        bool[] pol = new bool[3];
                        int[] ofs = new int[3];
                        int[] amp = new int[3];

                        try
                        {
                            for (int i = 0; i < 3; i++)
                            {
                                pol[i] = (data[2 + i * 5] == '-');
                                ofs[i] = Convert.ToInt32(strData.Substring(3 + i * 5, 2), 16);
                                if (ofs[i] >= 128) ofs[i] -= 256;
                                amp[i] = Convert.ToInt32(strData.Substring(5 + i * 5, 2), 16);
                            }
                            this.BeginInvoke((Action)(() => {
                                checkPol0.Checked = pol[0];
                                checkPol1.Checked = pol[1];
                                checkPol2.Checked = pol[2];
                                trackOfs0.Value = ofs[0];
                                trackOfs1.Value = ofs[1];
                                trackOfs2.Value = ofs[2];
                                trackAmp0.Value = amp[0];
                                trackAmp1.Value = amp[1];
                                trackAmp2.Value = amp[2];
                                textOfs0.Text = ofs[0].ToString();
                                textOfs1.Text = ofs[1].ToString();
                                textOfs2.Text = ofs[2].ToString();
                                textAmp0.Text = amp[0].ToString();
                                textAmp1.Text = amp[1].ToString();
                                textAmp2.Text = amp[2].ToString();
                            }));
                        }
                        catch (Exception ex)
                        {
                            ;// 何もしない
                        }
                    }
                    break;
                // バッテリー電圧
                case 'B':
                    int adc = Convert.ToInt32(strData.Substring(1, 3), 16);
                    double voltage = adc * 2 * 3.3 / 1024;
                    this.BeginInvoke((Action)(() =>
                    {
                        textBattery.Text = voltage.ToString("F2");
                    }));
                    break;
            }
        }

        // COMポート接続時
        private void onOpened(object sender, ComPortEventArgs e)
        {
            comPort.sendMessage("AL");
        }

        // COMポート切断時
        private void onClosed(object sender, ComPortEventArgs e)
        {
            textBattery.Text = "";
        }
    }
}
